// Copyright (c) Suikan
// Signal Format is 16bit signed fixed point number
#include "aduc7026.h"
#include "sine.h"
#include "pow2.h"
#include "pow10.h"
#include "synthesizer.h"

	/* GPIOprbg}XN */
#define BIT_LED				128




short adc[16];

int tone_index = 0;
int tone[]={
	987.768*Hz,
	987.768*Hz,
	1318.512*Hz,
	1318.512*Hz,
	1479.976*Hz,
	1318.512*Hz,
	1244.508*Hz,
	1108.732*Hz,
	987.768*Hz,
	830.61*Hz,
	987.768*Hz,
	1318.512*Hz,
	1479.976*Hz,
	1661.22*Hz,
	1975.536*Hz,
	1479.976*Hz,
	
	987.768*Hz,
	987.768*Hz,
	1318.512*Hz,
	1318.512*Hz,
	1479.976*Hz,
	1318.512*Hz,
	1244.508*Hz,
	1108.732*Hz,
	987.768*Hz,
	1479.976*Hz,
	1479.976*Hz,
	1479.976*Hz,
	1661.22*Hz,
	1318.512*Hz,
	
	1975.536*Hz,
	1975.536*Hz,
	1975.536*Hz,
	1661.22*Hz,
	1760*Hz,
	1760*Hz,
	1760*Hz,
	1479.976*Hz,
	1661.22*Hz,
	1661.22*Hz,
	1108.732*Hz,
	1479.976*Hz,
	987.768*Hz,
	
	987.768*Hz,
	987.768*Hz,
	1318.512*Hz,
	1318.512*Hz,
	1479.976*Hz,
	1318.512*Hz,
	1108.732*Hz,
	1244.508*Hz,
	987.768*Hz,
	1479.976*Hz,
	1661.22*Hz,
	1760*Hz,
	1479.976*Hz,
	1318.512*Hz
};


struct SYNTHESIZER{
	struct VCO{
		unsigned int phase, freq;
		short waveform;
		int duty;
		int amp;
	}vco, lfo;
	struct PATCH{
		short mode;
		int freq_gain, freq_offset;
		int amp_gain, amp_offset;
	}patch;
	struct EG{
		int a,d,s,r;			// value for each processing
		int state;			// ADSR state of EG.
		int coeff, offset;
		unsigned int t;			// tick by sample.
		int last_sutain_level;
		int attack_peak;
	}eg;	
	struct VCA{
		int gain;
	}vca;
	struct VCF{
		int frequency, resonance;	  	// 0 .. 1
		int b0, b1, b2, b3, b4;  //filter buffers (17.15 format )
		int p, q, f;
	}vcf;
	struct NOTE{
		 short on, off, state;
	}note;
} synth;

int main() 
{
	synth.vco.phase = 0;
	synth.vco.freq = tone[tone_index];
	synth.vco.waveform = WAVEFORM_SINE;
	synth.vco.amp = 0x7FFF;
	synth.lfo.freq = 0;
	synth.lfo.amp = 0;
	synth.lfo.phase = 0;
	synth.patch.mode = PATCH_NONE;
	synth.patch.freq_gain = 	0;			// 0
	synth.patch.freq_offset = 	_F(0.5);	// 0.5
	synth.patch.amp_gain = 		0;			// 0
	synth.patch.amp_offset = 	_F(0.5);	// 0.5
	synth.note.on = 0;
	synth.note.off = 0;
	synth.note.state = 0;
	synth.eg.t = 0;
	synth.eg.state = EG_STATE_NONE;
	synth.vca.gain = 0;
	synth.vcf.b0 = 0;
	synth.vcf.b1 = 0;
	synth.vcf.b2 = 0;
	synth.vcf.b3 = 0;
	synth.vcf.b4 = 0;
	synth.vcf.p = 0;
	synth.vcf.q = 0;
	synth.vcf.f = 0;

	
	setup_clock();
	setup_interrupt();
	setup_timer();
	setup_DAC();
	setup_ADC();
	setup_GPIO();
	
	while( 1 ){
		wait_timer();
		
		put_DAC( do_synthesis() );
		update_ADC();
		update_switch();
		update_flter();
		put_load();
	}
}

int do_synthesis(void) __ram
{
	int signal, sig_lfo, i;

	switch ( synth.vco.waveform ){
		case WAVEFORM_SINE:
			signal = sine(synth.vco.phase);
			break;
		case WAVEFORM_SAW:
			signal = synth.vco.phase >> 16;
			break;
		case WAVEFORM_SQUARE:
			signal = ((synth.vco.phase >>16)> synth.vco.duty )? _F(1):_F(-1) ;
			break;
		case WAVEFORM_NOISE:
			signal = 0;
			for ( i=0; i<8; i++){
				if ( synth.vco.phase & 0x1 ){
					synth.vco.phase >>= 1;
					synth.vco.phase ^= 0x80200003;
				}
				else
					synth.vco.phase >>= 1;
				signal += (int)synth.vco.phase >> 16;
			}
			if ( signal >= _F(1) )
				signal = 32767;
			else if ( _F(-1) > signal )
				signal = _F(-1);
			break;
	}	
	sig_lfo = sine( synth.lfo.phase )* synth.lfo.amp >> 15;
	

	{		 // Phase update of VCO by long long int.
		long long int freq;

		freq = synth.patch.freq_gain * sig_lfo >> 16;
		freq += _F(0.5);
		freq *= (long long int)synth.vco.freq;
		freq >>= 15;
		synth.vco.phase += freq;
	}

	synth.lfo.phase += synth.lfo.freq;

	signal = do_filter( signal>>2 );

	update_EG();
	signal = signal * synth.vca.gain >> 15;		
	
	{
		int amplitude;
		
		amplitude = synth.patch.amp_offset;
		amplitude += synth.patch.amp_gain * sig_lfo >> 15;
		signal = signal * amplitude >> 15;
	}
	return( signal );
}

void update_EG(void) __ram
{
		if ( synth.note.on ){
			synth.eg.state = EG_STATE_A;
			synth.eg.t = 0;
			synth.eg.coeff = _F(1) - synth.vca.gain;	// inherit current enverlop for smooth transition
		}

		if ( synth.eg.state == EG_STATE_A ){
			short gain;
			
			gain = POW2( synth.eg.t ) * synth.eg.coeff >> 15;
			synth.vca.gain = _F(1) - gain;
			if ( synth.vca.gain >= MAX_ATTACK ){
				synth.eg.t = 0;	 
				synth.eg.attack_peak = synth.vca.gain;
				if ( synth.note.state ){	// note is still on
					synth.eg.state = EG_STATE_D;
					synth.eg.last_sutain_level = synth.eg.s;
					synth.eg.offset = synth.eg.last_sutain_level * synth.eg.attack_peak >> 15;
					synth.eg.coeff = synth.eg.attack_peak - synth.eg.offset;
				}
				else {						// note is already off
					synth.eg.state = EG_STATE_R;
					synth.eg.coeff = synth.eg.attack_peak;
				};		
			}
			else
				synth.eg.t += synth.eg.a;
		}
		else if ( synth.eg.state == EG_STATE_D ){
			short gain;
	
			if ( synth.eg.last_sutain_level != synth.eg.s ){
				synth.eg.t = synth.eg.d;	 
				synth.eg.last_sutain_level = synth.eg.s;
				synth.eg.offset = synth.eg.last_sutain_level * synth.eg.attack_peak >> 15;
				synth.eg.coeff = synth.vca.gain - synth.eg.offset;
			}
			gain = POW2( synth.eg.t );
			synth.vca.gain = ( synth.eg.coeff * gain >> 15 )+ synth.eg.offset;
			if ( synth.note.off ){
				synth.eg.state = EG_STATE_R;
				synth.eg.t = 0;	
				synth.eg.coeff = synth.vca.gain;
			}
			else if ( gain < 1 )
				synth.eg.state = EG_STATE_S; 
			else 
				synth.eg.t += synth.eg.d;
		}
		else if ( synth.eg.state == EG_STATE_S ) {
			short gain;
			
			gain = synth.eg.attack_peak * synth.eg.s >> 15;
			synth.vca.gain = gain;
			if ( synth.note.off ){
				synth.eg.state = EG_STATE_R;
				synth.eg.t = 0;	
				synth.eg.coeff = synth.vca.gain;
			}
		}
		else if ( synth.eg.state == EG_STATE_R ) {
			short gain;
			
			gain = POW2( synth.eg.t );
			synth.vca.gain = synth.eg.coeff * gain >> 15;
			if ( gain < 1 )
				synth.eg.state = EG_STATE_NONE; 
			else
				synth.eg.t += synth.eg.r;
		}
}

void setup_clock( void )
{
	POWKEY1 = 0x01;
	POWCON = 0;		// set frequency 41.78MHz
	POWKEY2 = 0xF4;
}

void setup_interrupt( void )
{
	IRQEN = 0;		// disable all interrupt
}

void LED_on( void ) __ram
{
	GP4CLR = GPIO_WRITE(BIT_LED);	/* Turn LED on */
}

void LED_off( void ) __ram
{
	GP4SET = GPIO_WRITE(BIT_LED);	/* Turn LED off */
}

/*
 *  Setup GPIO/SPI port of ADuC7026. Note that GP2 bit 0 is assinge to select
 *  MAX7221
 */
void setup_GPIO( void )
{
	GP1CON = 0x00000000;	/* SPICLK, SPIMISO, SPIMOSI, SPICSL I */
	GP2CON = 0x00000000;	/* SsGPIO */
	GP3CON = 0x00000000;	/* SsGPIO */
	GP4CON = 0x00000000;	/* SsGPIO */
	
	GP1DAT = GPIO_DIR(0xC0);
	GP2DAT = GPIO_DIR(0x86);
	GP3DAT = GPIO_DIR(0xC0);
	GP4DAT = GPIO_DIR(BIT_LED | 0x01);
	
	GP4SET = GPIO_WRITE(BIT_LED);				/* LED OFF */
	GPSET(LED_PATCH_VCA);		// setup LED
	GPSET(LED_PATCH_RING);
	GPSET(LED_PATCH_VCO);
	GPSET(LED_PATCH_VCF);
	GPCLR(LED_WAVEFORM_SINE);
	GPSET(LED_WAVEFORM_SQUARE);
	GPSET(LED_WAVEFORM_SAW);
	GPSET(LED_WAVEFORM_NOISE);
}

void setup_timer( void )
{
	T1LD = TIMER1DIVISOR;	
	T1CON = 0x00C0;		/* HCLK/1, Count Down, Enable, Periodic */
}

void wait_timer( void ) __ram
{
	while ( !( IRQSIG & 0x0008 ) )	/* Have timer 1 interupt happen ? */
		;
	T1CLRI = 0;			/* Clear Interrupt */
}

void setup_DAC( void )
{
	DAC0CON = 0x33;	/* Enable, update by timer1, scale to AVDD */
	DAC3CON = 0x33;	/* Enable, update by timer1, scale to AVDD */
}

void put_DAC( short value )	__ram
{
	/* ajust signal 2.5V center,  0.33* full scale */
	DAC3DAT = ( value + 49647 ) << 12;
}

void put_load( void ) __ram
{
		// get the current timer value, and calcrate the 
		// left time to expire.
		// 4096 is the full scale fuctor. TIMER1DIVISOR is the period.
		// DAC0 output represent the CPU load percentage.
	DAC0DAT = T1VAL * (65536*4096 / TIMER1DIVISOR );
}

void setup_ADC(void)
{
	REFCON = 1;
	ADCCP = 0;		/* Start Channel is 0 */
	ADCCON = 0x0A1;	/*Default ADC speed, enable conversion, Single-end, Timer1 Driven */
}

/* Call this routine after all processing */
void update_ADC(void) __ram
{
	static channel = 0;
	
	adc[channel] = (ADCDAT >> 13)&0x7FFF;		/* ADC data is located at bit27:16, put it into bit15:0 */
	switch ( channel ){
		case VOL_EG_A : 
			synth.eg.a = TICKMAX * pow10tbl[(32767-adc[VOL_EG_A])>>5];
			break;
		case VOL_EG_D : 
			synth.eg.d = TICKMAX * pow10tbl[(32767-adc[VOL_EG_D])>>5];
			break;
		case VOL_EG_S : 
			synth.eg.s = pow10tbl[(adc[VOL_EG_S]>>6) + 512]; 	// map 16bit signed int to 2dicade.
			break;
		case VOL_EG_R : 
			synth.eg.r = TICKMAX * pow10tbl[(32767-adc[VOL_EG_R])>>5];
			break;
		case VOL_LFO_AMP : 
			synth.lfo.amp = pow10tbl[(adc[VOL_LFO_AMP]>>6) + 512];
			break;
		case VOL_LFO_FREQ : 
			synth.lfo.freq = pow10tbl[(adc[VOL_LFO_FREQ]>>6) + 512] * ( 10*Hz / 32768 );
			break;
		case VOL_VCF_RES : 
			synth.vcf.resonance = adc[VOL_VCF_RES];
			break;
		case VOL_VCF_FREQ : 
			synth.vcf.frequency = adc[VOL_VCF_FREQ];
			break;
		case VOL_VCO_DUTY :
			synth.vco.duty = adc[VOL_VCO_DUTY];
			break;
	};
	channel = ( channel + 1 ) & 0x0F;	/* Select next MUX */
	ADCCP = channel;					/* Set next channel */
										/* Conversion is done by timer1 */
}

void update_switch(void) __ram
{
	static short waveform_deadcount = 0;
	static short waveform_switch = 0;
	static short patch_deadcount = 0;
	static short patch_switch = 0;
	static short note_deadcount = 0;
	static short note_switch = 0;
	unsigned short sw;

	sw = GPTST(SWITCH_NOTE);
	synth.note.off = 0;
	synth.note.on = 0;		
	if ( note_deadcount ){
		note_deadcount --;
	}
	else if ( sw && ! note_switch ){
		note_deadcount = DEAD_TIME;
		note_switch = 1;
		synth.note.state = 0;
		synth.note.off = 1;
	}
	else if ( !sw && note_switch ){
		note_deadcount = DEAD_TIME;
		note_switch = 0;
		synth.note.state = 1;
		synth.note.on = 1;
				/* Next tone */
		synth.vco.freq = tone[tone_index];
		++tone_index;
		if ( tone_index >= 57 )
			tone_index = 0;
	}
	
	
	sw = GPTST(SWITCH_PATCH);
	if ( patch_deadcount )
		patch_deadcount --;
	else if ( sw && ! patch_switch || !sw && patch_switch ){
		patch_deadcount = DEAD_TIME;
		patch_switch = !patch_switch;
		if ( !patch_switch ){
			GPSET(LED_PATCH_VCA);		// turn all PATCH switch off;
			GPSET(LED_PATCH_RING);
			GPSET(LED_PATCH_VCO);
			GPSET(LED_PATCH_VCF);
			synth.patch.mode ++;
			if ( synth.patch.mode > PATCH_VCF ) 
				synth.patch.mode = PATCH_NONE;

			if ( synth.patch.mode == PATCH_NONE ){
				synth.patch.freq_gain = 	0;			// 0
				synth.patch.freq_offset = 	_F(0.5);	// 0.5
				synth.patch.amp_gain = 		0;			// 0
				synth.patch.amp_offset = 	_F(0.5);	// 0.5
			}
			else if ( synth.patch.mode == PATCH_VCA ){
				GPCLR(LED_PATCH_VCA);
				synth.patch.freq_gain = 	0;			// 0
				synth.patch.freq_offset = 	_F(0.5);	// 0.5
				synth.patch.amp_gain = 		_F(0.5);	// 0.5
				synth.patch.amp_offset = 	_F(0.5);	// 0.5
			}
			else if ( synth.patch.mode == PATCH_RING ){
				GPCLR(LED_PATCH_RING);
				synth.patch.freq_gain = 	0;			// 0
				synth.patch.freq_offset = 	_F(0.5);	// 0.5
				synth.patch.amp_gain = 		_F(1);		// 0.999
				synth.patch.amp_offset = 	0;			// 0
			}
			else if ( synth.patch.mode == PATCH_VCO ){
				GPCLR(LED_PATCH_VCO);
				synth.patch.freq_gain = 	_F(0.5);	// 0.5
				synth.patch.freq_offset = 	_F(0.5);	// 0.5
				synth.patch.amp_gain = 		0;			// 0
				synth.patch.amp_offset = 	_F(1);		// 0.999
			}
			else if ( synth.patch.mode == PATCH_VCF ){
				GPCLR(LED_PATCH_VCF);		
				synth.patch.freq_gain = 	0;			// 0
				synth.patch.freq_offset = 	_F(0.5);	// 0.5
				synth.patch.amp_gain = 		0;			// 0
				synth.patch.amp_offset = 	_F(1);		// 0.999
			}
		}
	}

	
	sw = GPTST(SWITCH_WAVEFORM);
	if ( waveform_deadcount )
		waveform_deadcount --;
	else if ( sw && ! waveform_switch || !sw && waveform_switch ){
		waveform_deadcount = DEAD_TIME;
		waveform_switch = !waveform_switch;
		if ( !waveform_switch ){
			GPSET(LED_WAVEFORM_SINE);		// turn all waveform switch off;
			GPSET(LED_WAVEFORM_SQUARE);
			GPSET(LED_WAVEFORM_SAW);
			GPSET(LED_WAVEFORM_NOISE);
			synth.vco.waveform ++;
			if ( synth.vco.waveform > WAVEFORM_NOISE ) 
				synth.vco.waveform = WAVEFORM_SINE;
			if ( synth.vco.waveform == WAVEFORM_SINE ){
				GPCLR(LED_WAVEFORM_SINE);
			}
			else if ( synth.vco.waveform == WAVEFORM_SQUARE ){
				GPCLR(LED_WAVEFORM_SQUARE);
			}
			else if ( synth.vco.waveform == WAVEFORM_SAW ){
				GPCLR(LED_WAVEFORM_SAW);
			}
			else if ( synth.vco.waveform == WAVEFORM_NOISE ){
				GPCLR(LED_WAVEFORM_NOISE);		
			}
		}
	}
}

/*
*	Up-to-date the filter coefficient. 
*/
void update_flter(void)	__ram
{
//   Code :	http://www.musicdsp.org/archive.php?classid=3#24
// Moog 24 dB/oct resonant lowpass VCF
// References: CSound source code, Stilson/Smith CCRMA paper.
// Modified by paul.kellett@maxim.abel.co.uk July 2000
  int f, p, q;             //filter coefficients

// Set coefficients given frequency & resonance [0.0...1.0]

  q = _F(1.0) - synth.vcf.frequency;
  p = synth.vcf.frequency + fxmul( _F(0.8), fxmul(synth.vcf.frequency, q) );
  f = p + p - _F(1.0);
  q = fxmul( synth.vcf.resonance, _F(1.0) + fxmul( fxmul(_F(0.5), q ), ( _F(1.0) - q + fxmul(_F(5.6), fxmul(q, q) ) ) ) );
  synth.vcf.q = q;
  synth.vcf.p = p;
  synth.vcf.f = f;
}

/*
*	Filter processing the received parameter sample.
*/

int do_filter( int input ) __ram
{
	int t1, t2;              //temporary buffers

  	input -= fxmulEX( synth.vcf.q, synth.vcf.b4 );                          //feedback
  	t1 = synth.vcf.b1;  	synth.vcf.b1 = fxmulEX( (input + synth.vcf.b0) , synth.vcf.p ) - fxmulEX( synth.vcf.b1, synth.vcf.f );
  	t2 = synth.vcf.b2;  	synth.vcf.b2 = fxmulEX( (synth.vcf.b1 + t1)    , synth.vcf.p ) - fxmulEX( synth.vcf.b2, synth.vcf.f );
  	t1 = synth.vcf.b3;  	synth.vcf.b3 = fxmulEX( (synth.vcf.b2 + t2)    , synth.vcf.p ) - fxmulEX( synth.vcf.b3, synth.vcf.f );
    			        	synth.vcf.b4 = fxmulEX( (synth.vcf.b3 + t1)    , synth.vcf.p ) - fxmulEX( synth.vcf.b4, synth.vcf.f );
  	synth.vcf.b4 = synth.vcf.b4 - fxmulEX( fxmulEX( synth.vcf.b4, synth.vcf.b4 ), fxmulEX( synth.vcf.b4, _F(0.166667f) ) );    //clipping
  	synth.vcf.b0 = input;

	return ( synth.vcf.b4 );// Lowpass  output:  b4
							// Highpass output:  in - b4;
							// Bandpass output:  3.0f * (b3 - b4);
}

/*
* 	Multiply 17.15 x 17.15 format fixed point
*/
/*
int fxmulEX( int para, int parb) __ram
{
	__asm{
		ldav	r2, para
		ldav	r1, parb
		smull	r3,r4,r1,r2
		mov		r0,r4,lsr #15
		orr		r0,r0,r3, lsl #17
	}
	// return value is placed on R0
} 
*/