#ifndef SYNTHESIZER_H_
#define SYNTHESIZER_H_

#define SAMPLEFREQ 24000
#define TIMER1DIVISOR (41779200/SAMPLEFREQ)
#define Hz ((int)(4294967296LL/SAMPLEFREQ))
#define FREQ_A		  (880*Hz)
#define MAX_ATTACK	(32767 - 8192)
/*
 * TICKMAX is given by (2^26*10000/sample>>15)
 * It provide 10e-4 sec when you multiply by 0x7fff ( 0.999... by fixed point )
 */
#define TICKMAX ((67108864*10)/(SAMPLEFREQ/1000)>>15)	

	/* Macro for fixed point immediate */
#define _F(x) ((int)(x*32768))
#define fxmul(x,y) ((x)*(y)>>15)


	/* GPIO_DATtB[hANZX}N */
#define GPIO_DIR( x ) (x<<24)
#define GPIO_WRITE( x ) (x<<16)
#define GPIO_STRAP( x ) (x<<8)
#define GPIO_READ( x )	(x)

#define WAVEFORM_SINE 			0
#define WAVEFORM_SQUARE 		1
#define WAVEFORM_SAW 			2
#define WAVEFORM_NOISE			3

#define PATCH_NONE		0
#define PATCH_VCA		1
#define PATCH_RING		2
#define PATCH_VCO		3
#define PATCH_VCF		4

#define DEAD_TIME 120

#define BIT_SWITCH_WAVEFORM		(1<<1)
#define BIT_LED_WAVEFORM_SINE	(1<<6)
#define BIT_LED_WAVEFORM_SQUARE	(1<<2)
#define BIT_LED_WAVEFORM_SAW	(1<<7)
#define BIT_LED_WAVEFORM_NOISE	(1<<6)

#define PORT_SWITCH_WAVEFORM		4
#define PORT_LED_WAVEFORM_SINE		1
#define PORT_LED_WAVEFORM_SQUARE	2
#define PORT_LED_WAVEFORM_SAW		2
#define PORT_LED_WAVEFORM_NOISE		3

#define BIT_SWITCH_PATCH	(1<<5)
#define BIT_LED_PATCH_VCA	(1<<0)
#define BIT_LED_PATCH_RING	(1<<7)
#define BIT_LED_PATCH_VCO	(1<<1)
#define BIT_LED_PATCH_VCF	(1<<7)

#define PORT_SWITCH_PATCH	1
#define PORT_LED_PATCH_VCA	4
#define PORT_LED_PATCH_RING	1
#define PORT_LED_PATCH_VCO	2
#define PORT_LED_PATCH_VCF	3

#define BIT_SWITCH_NOTE		(1<<0)
#define PORT_SWITCH_NOTE	0

#define GPSET(bit) (&GP0SET)[PORT_##bit*4] = GPIO_WRITE(BIT_##bit)
#define GPCLR(bit) (&GP0CLR)[PORT_##bit*4] = GPIO_WRITE(BIT_##bit)
#define GPTST(bit) ((&GP0DAT)[PORT_##bit*4] & GPIO_READ(BIT_##bit))

#define VOL_EG_A	9
#define VOL_EG_D	7
#define VOL_EG_S	5
#define VOL_EG_R	1

#define VOL_LFO_FREQ	10
#define VOL_LFO_AMP		8
#define VOL_VCF_FREQ	6
#define VOL_VCF_RES		4

#define VOL_VCO_DUTY	0

#define EG_STATE_NONE	0
#define EG_STATE_A		1
#define EG_STATE_D		2
#define EG_STATE_S		3
#define EG_STATE_R		4


void setup_GPIO( void );
void setup_interrupt( void );
void setup_clock( void );
void setup_timer( void );
void setup_DAC(void);
void put_DAC( short );
void put_load( void );
void LED_on( void );
void LED_off( void );
void wait_timer( void );
void setup_ADC(void);
void update_ADC(void);
void update_switch(void);
void update_EG(void);
int do_synthesis(void);
void update_flter(void);
int do_filter( int input );
//int fxmulEX( int para, int parb );
#define fxmulEX(x,y) fxmul(x,y)

#endif /*SYNTHESIZER_H_*/
